package org.gudenau.cooler;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.block.Block;
import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.material.Material;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemDye;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.Icon;
import net.minecraft.util.MathHelper;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeDirection;

public class BlockCooler extends Block {
	
	public BlockCooler(int id) {
		super(id, Material.cloth);

		setUnlocalizedName("gud_cooler_block_cooler");
		setCreativeTab(CreativeTabs.tabFood);
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void getSubBlocks(int par1, CreativeTabs tab,
			List items) {
		ItemStack stack = new ItemStack(this, 1, 0xFFFFFF);
		
		stack.stackTagCompound = new NBTTagCompound();
		items.add(stack.copy());
		stack.setItemDamage(0x1FFFFFF);
		stack.stackTagCompound.setBoolean("clear", true);
		items.add(stack.copy());
	}

	@Override
	public int getRenderType() {
		return Cooler.proxy.renderId;
	}

	@Override
	@SideOnly(Side.CLIENT)
	public Icon getBlockTexture(IBlockAccess world, int x, int y, int z,
			int side) {
		return Block.blockSnow.getBlockTexture(world, z, y, z, side);
	}

	@Override
	public ItemStack getPickBlock(MovingObjectPosition target, World world,
			int x, int y, int z) {
		ItemStack stack = new ItemStack(this, 1, 0);
		NBTTagCompound tag = new NBTTagCompound();

		TileEntity tile = world.getBlockTileEntity(x, y, z);

		if (tile != null && tile instanceof TileCooler) {
			TileCooler cooler = ((TileCooler) tile);
			cooler.writeToItem(tag);
			stack.setItemDamage((cooler.clear ? 0x1000000 : 0)
					+ (cooler.red & 0xFF) + ((cooler.green & 0xFF) << 8)
					+ ((cooler.blue & 0xFF) << 16));
		}

		stack.stackTagCompound = tag;

		return stack;
	}

	@Override
	public int quantityDropped(Random par1Random) {
		return 0;
	}

	@Override
	public void breakBlock(World world, int x, int y, int z, int oldId,
			int oldMeta) {
		TileEntity tile = world.getBlockTileEntity(x, y, z);

		if (!world.getClosestPlayer(x + 0.5, y + 0.5, z + 0.5, 8).capabilities.isCreativeMode) {

			if (tile != null && tile instanceof TileCooler) {
				ItemStack stack = new ItemStack(this, 1, 0);
				NBTTagCompound tag = new NBTTagCompound();
				TileCooler cooler = ((TileCooler) tile);
				cooler.writeToItem(tag);
				tag.removeTag("handle");
				stack.stackTagCompound = tag;
				stack.setItemDamage((cooler.clear ? 0x1000000 : 0)
						+ (cooler.red & 0xFF) + ((cooler.green & 0xFF) << 8)
						+ ((cooler.blue & 0xFF) << 16));
				dropBlockAsItem_do(world, x, y, z, stack);
			}
		}

		world.removeBlockTileEntity(x, y, z);
	}

	@Override
	public boolean onBlockActivated(World world, int x, int y, int z,
			EntityPlayer player, int side, float hitX, float hitY, float hitZ) {
		if (player.getHeldItem() != null
				&& player.getHeldItem().getItem() != null
				&& player.getHeldItem().getItem() instanceof ItemDye) {
			TileEntity tile = world.getBlockTileEntity(x, y, z);

			if (tile instanceof TileCooler) {
				TileCooler cooler = (TileCooler) tile;

				cooler.red = (ItemDye.dyeColors[player.getHeldItem()
						.getItemDamage()] >> 16) & 0xFF;
				cooler.green = (ItemDye.dyeColors[player.getHeldItem()
						.getItemDamage()] >> 8) & 0xFF;
				cooler.blue = ItemDye.dyeColors[player.getHeldItem()
						.getItemDamage()] & 0xFF;
				return true;
			}
		}

		player.openGui(Cooler.instance, 0, world, x, y, z);
		return true;
	}

	@SideOnly(Side.CLIENT)
	@Override
	public boolean shouldSideBeRendered(IBlockAccess world, int x, int y,
			int z, int side) {
		return false;
	}

	@Override
	public boolean isOpaqueCube() {
		return false;
	}

	@Override
	public boolean renderAsNormalBlock() {
		return false;
	}

	@Override
	public boolean hasTileEntity(int meta) {
		return true;
	}

	@Override
	public TileEntity createTileEntity(World world, int metadata) {
		return new TileCooler();
	}

	@Override
	public void setBlockBoundsBasedOnState(IBlockAccess world, int x, int y,
			int z) {
		int meta = world.getBlockMetadata(x, y, z) & 3;

		switch (meta) {
		case 0:
		case 2:
			setBlockBounds(0.28125F, 0.0F, 0.0625F, 0.71875F, 0.6875F, 0.9375F);
			break;
		case 1:
		case 3:
		default:
			setBlockBounds(0.0625F, 0.0F, 0.28125F, 0.9375F, 0.6875F, 0.71875F);
			break;
		}
	}

	@Override
	public void setBlockBoundsForItemRender() {
		setBlockBounds(0.0625F, 0.0F, 0.28125F, 0.9375F, 0.6875F, 0.71875F);
	}

	@Override
	public void onBlockAdded(World world, int x, int y, int z) {
		super.onBlockAdded(world, x, y, z);
		this.setDefaultDirection(world, x, y, z);
	}

	private void setDefaultDirection(World world, int x, int y, int z) {
		if (!world.isRemote) {
			int l = world.getBlockId(x, y, z - 1);
			int i1 = world.getBlockId(x, y, z + 1);
			int j1 = world.getBlockId(x - 1, y, z);
			int k1 = world.getBlockId(x + 1, y, z);
			byte facing = 2;

			if (Block.opaqueCubeLookup[l] && !Block.opaqueCubeLookup[i1]) {
				facing = 1;
			}

			if (Block.opaqueCubeLookup[i1] && !Block.opaqueCubeLookup[l]) {
				facing = 3;
			}

			if (Block.opaqueCubeLookup[j1] && !Block.opaqueCubeLookup[k1]) {
				facing = 2;
			}

			if (Block.opaqueCubeLookup[k1] && !Block.opaqueCubeLookup[j1]) {
				facing = 0;
			}

			world.setBlockMetadataWithNotify(x, y, z, facing, 2);
		}
	}

	@Override
	public void onBlockPlacedBy(World world, int x, int y, int z,
			EntityLivingBase entity, ItemStack stack) {
		int facing = MathHelper
				.floor_double((double) (entity.rotationYaw * 4.0F / 360.0F) + 0.5D) & 3;

		if (facing == 0) {
			facing = 3;
		} else if (facing == 1) {
			facing = 2;
		} else if (facing == 2) {
			facing = 1;
		} else if (facing == 3) {
			facing = 0;
		}

		world.setBlockMetadataWithNotify(x, y, z, facing, 2);
	}

	@Override
	public boolean canPlaceBlockAt(World world, int x, int y, int z) {
		return super.canPlaceBlockAt(world, x, y, z)
				&& canBlockStay(world, x, y, z);
	}

	@Override
	public void onNeighborBlockChange(World world, int x, int y, int z, int id) {
		if (!canBlockStay(world, x, y, z)) {
			dropBlockAsItem(world, x, y, z, 0, 0);
			world.setBlockToAir(x, y, z);
		}
	}

	@Override
	public boolean canBlockStay(World world, int x, int y, int z) {
		return world.isBlockSolidOnSide(x, y - 1, z, ForgeDirection.NORTH);
	}

	@Override
	public boolean onBlockEventReceived(World world, int x, int y, int z,
			int id, int event) {
		super.onBlockEventReceived(world, x, y, z, id, event);
		TileEntity tileentity = world.getBlockTileEntity(x, y, z);
		return tileentity != null ? tileentity.receiveClientEvent(id, event)
				: false;
	}
}
