package org.gudenau.cooler;

import java.util.List;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.FoodStats;
import net.minecraft.util.Icon;
import net.minecraft.util.StatCollector;
import net.minecraft.world.World;

public class ItemCooler extends ItemBlock {

	public ItemCooler(int id) {
		super(id);
		setHasSubtypes(true);
	}

	@SideOnly(Side.CLIENT)
	public void addInformation(ItemStack stack, EntityPlayer player, List info,
			boolean par4) {
		if (!stack.hasTagCompound()) {
			return;
		}

		NBTTagCompound tag = stack.stackTagCompound;

		if (tag.hasKey("buffer")) {
			info.add("Buffer: " + String.format("%.2f", tag.getFloat("buffer")));
		}

		if (tag.hasKey("items")) {
			NBTTagCompound tags = tag.getCompoundTag("items");

			for (int i = 0; i < 6; i++) {
				if (tags.hasKey("" + i)) {
					ItemStack item = ItemStack.loadItemStackFromNBT(tags
							.getCompoundTag("" + i));
					info.add(StatCollector.translateToLocal(item
							.getUnlocalizedName() + ".name")
							+ (item.stackSize != 1 ? "x" + item.stackSize : ""));
				}
			}
		}

		if (tag.hasKey("red") || tag.hasKey("green") || tag.hasKey("blue")) {
			info.add("Colored");
		}

		if (tag.hasKey("clear") && tag.getBoolean("clear")) {
			info.add("Clear");
		}
	}

	@Override
	public boolean placeBlockAt(ItemStack stack, EntityPlayer player,
			World world, int x, int y, int z, int side, float hitX, float hitY,
			float hitZ, int metadata) {
		boolean orig = super.placeBlockAt(stack, player, world, x, y, z, side,
				hitX, hitY, hitZ, metadata);

		if (orig && stack.hasTagCompound()) {
			TileCooler tile = new TileCooler();
			tile.readFromItem(stack.stackTagCompound);
			world.setBlockTileEntity(x, y, z, tile);
		}

		return orig;
	}

	@Override
	public void onUpdate(ItemStack stack, World world, Entity entity, int par4,
			boolean par5) {
		if (!stack.hasTagCompound()) {
			stack.stackTagCompound = new NBTTagCompound();
			return;
		}

		if (!(entity instanceof EntityPlayer)) {
			return;
		}

		NBTTagCompound tag = stack.stackTagCompound;

		EntityPlayer player = (EntityPlayer) entity;
		FoodStats food = player.getFoodStats();

		if (!player.capabilities.disableDamage && food.needFood()
				& stack.stackTagCompound.hasKey("items")) {
			if (Cooler.buffer) {
				float sat = food.getSaturationLevel();
				int hunger = food.getFoodLevel();
				float buffer = 0;

				if (tag.hasKey("buffer")) {
					buffer = tag.getFloat("buffer");
				}

				if (hunger < 20) {
					buffer -= (20 - hunger);
					hunger = 20;
				}

				if (sat < 20) {
					sat = 20;
				}

				float avalible = 0;

				ItemStack[] items = new ItemStack[6];

				if (tag.hasKey("items")) {
					NBTTagCompound tags = tag.getCompoundTag("items");

					for (int i = 0; i < 6; i++) {
						if (tags.hasKey("" + i)) {
							items[i] = ItemStack.loadItemStackFromNBT(tags
									.getCompoundTag("" + i));
						}
					}
				}

				if (buffer < 0) {
					world.playSoundAtEntity(player, "random.burp", 0.5F,
							world.rand.nextFloat() * 0.1F + 0.9F);
				}

				for (ItemStack s : items) {
					if (buffer >= 0) {
						break;
					}

					if (s == null || s.getItem() == null) {
						continue;
					}

					ItemFood item = ((ItemFood) s.getItem());

					float amount = (((float) item.getHealAmount())
							* item.getSaturationModifier() * 2F)
							+ (float) item.getHealAmount();

					while (s.stackSize > 0 && buffer < 0) {
						buffer += amount;
						s.stackSize--;
					}
				}

				NBTTagCompound tags = new NBTTagCompound();

				boolean stacks = false;
				for (int i = 0; i < 6; i++) {
					ItemStack stack2 = items[i];
					if (stack2 != null && stack2.stackSize > 0) {
						NBTTagCompound tmp = new NBTTagCompound();
						stack2.writeToNBT(tmp);
						tags.setTag("" + i, tmp);
						stacks = true;
					}
				}

				if (stacks) {
					tag.setTag("items", tags);
				}

				tag.setFloat("buffer", buffer);
				stack.stackTagCompound = tag;
				food.setFoodSaturationLevel(sat);
				food.setFoodLevel(hunger);
			} else {
				ItemStack[] items = new ItemStack[6];
				NBTTagCompound tags = tag.getCompoundTag("items");
				for (int i = 0; i < 6; i++) {
					if (tags.hasKey("" + i)) {
						items[i] = ItemStack.loadItemStackFromNBT(tags
								.getCompoundTag("" + i));
					}
				}

				for (ItemStack stack2 : items) {
					if (stack2 != null) {
						food.addStats((ItemFood) stack2.getItem());
						stack2.stackSize--;
						world.playSoundAtEntity(player, "random.burp", 0.5F,
								world.rand.nextFloat() * 0.1F + 0.9F);
						break;
					}
				}

				tags = new NBTTagCompound();

				boolean stacks = false;
				for (int i = 0; i < 6; i++) {
					ItemStack stack2 = items[i];
					if (stack2 != null && stack2.stackSize > 0) {
						NBTTagCompound tmp = new NBTTagCompound();
						stack2.writeToNBT(tmp);
						tags.setTag("" + i, tmp);
						stacks = true;
					}
				}

				if (stacks) {
					tag.setTag("items", tags);
				}
				stack.stackTagCompound = tag;
			}
		}
	}
}
