package org.gudenau.cooler;

import org.lwjgl.opengl.GL11;

import com.google.common.primitives.SignedBytes;

import cpw.mods.fml.client.registry.ISimpleBlockRenderingHandler;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.entity.RenderItem;
import net.minecraft.client.renderer.entity.RenderManager;
import net.minecraft.client.renderer.tileentity.TileEntityRenderer;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.IBlockAccess;

@SideOnly(Side.CLIENT)
public class RenderCooler extends TileEntitySpecialRenderer{

	private static final ResourceLocation texture = new ResourceLocation("gud_cooler:textures/blocks/cooler.png");
	private static final ResourceLocation textureClear = new ResourceLocation("gud_cooler:textures/blocks/cooler_clear.png");
	private static final ModelCooler model = new ModelCooler();
	private final RenderItem itemRenderer;
	
	public RenderCooler(){
		itemRenderer = new RenderItem() {
            @Override
            public byte getMiniBlockCount(ItemStack stack) {
                return SignedBytes.saturatedCast(Math.min(stack.stackSize / 32, 15) + 1);
            }
            @Override
            public byte getMiniItemCount(ItemStack stack) {
                return SignedBytes.saturatedCast(Math.min(stack.stackSize / 32, 7) + 1);
            }
            @Override
            public boolean shouldBob() {
                return false;
            }
            @Override
            public boolean shouldSpreadItems() {
                return false;
            }
        };
        itemRenderer.setRenderManager(RenderManager.instance);
	}
	
	@Override
	public void renderTileEntityAt(TileEntity tile, double x, double y,
			double z, float delta) {
		if(tile instanceof TileCooler){
			renderCooler((TileCooler)tile, x, y, z, delta);
		}
	}

	private void renderCooler(TileCooler tile, double x, double y, double z,
			float delta) {
		GL11.glPushMatrix();
		
		GL11.glTranslatef((float)x + 0.5F, (float)y + 0.5F, (float)z + 0.5F);
		int meta = tile.worldObj.getBlockMetadata(tile.xCoord, tile.yCoord, tile.zCoord) + 1 & 0x3;
		GL11.glRotatef(meta * 90, 0, 1, 0);
		GL11.glTranslatef(-0.5F, -0.5F, -0.65625F);
		
		bindTexture(tile.clear ? textureClear : texture);
		
		GL11.glColor3f(tile.red / 256F, tile.green / 256F, tile.blue / 256F);
		
		float f1 = tile.lastAngle + (tile.angle - tile.lastAngle) * delta;
        f1 = 1.0F - f1;
        f1 = 1.0F - f1 * f1 * f1;
        model.lid.rotateAngleX = -(f1 * (float)Math.PI / 2.0F);
        
        f1 = tile.lastAngle2 + (tile.angle2 - tile.lastAngle2) * delta;
        f1 = 1.0F - f1;
        f1 = 1.0F - f1 * f1 * f1;
        model.handle.rotateAngleX = (float)Math.PI / 1.3F * f1;
		model.renderAll();
		
		if(tile.clear){
			GL11.glColor3f(1.0F, 1.0F, 1.0F);
			GL11.glDisable(GL11.GL_LIGHTING);
			GL11.glDisable(GL11.GL_CULL_FACE);
			
			EntityItem entityItem = new EntityItem(tile.worldObj);
			entityItem.hoverStart = 0.0F;
			
			int slot = -1;
			
			GL11.glTranslatef(0.05F, -0.05F, 0.65F);
			
			for(ItemStack item : tile.getInventory()){
				slot++;
				if(item == null){
					continue;
				}
				
				GL11.glPushMatrix();
				GL11.glTranslatef((3 - (slot % 3)) / 4.5F, (2 - slot / 3) / 4.5F, 0);
				GL11.glRotatef(tile.itemRot + slot * 30, 0, 1, 0);
				GL11.glScalef(0.5F, 0.5F, 0.5F);
				entityItem.setEntityItemStack(item);
				itemRenderer.doRenderItem(entityItem, 0, 0, 0, 0, 0);
				GL11.glPopMatrix();
			}
		
			GL11.glEnable(GL11.GL_CULL_FACE);
			GL11.glEnable(GL11.GL_LIGHTING);
		}
		GL11.glPopMatrix();
	}

}
