package org.gudenau.cooler;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.network.INetworkManager;
import net.minecraft.network.packet.Packet;
import net.minecraft.network.packet.Packet132TileEntityData;
import net.minecraft.tileentity.TileEntity;

public class TileCooler extends TileEntity implements ISidedInventory {

	private int open = 0;
	public float angle = 0;
	public float angle2 = 0;
	
	public int red = 255, green = 255, blue = 255;

	private ItemStack[] items;
	private String name;

	private float buffer = 0.0F;
	public float lastAngle;
	public float lastAngle2;
	
	public boolean clear = false;
	public int itemRot = 0;
	
	public TileCooler() {
		items = new ItemStack[6];
	}

	@Override
	public void readFromNBT(NBTTagCompound tag) {
		super.readFromNBT(tag);
		readFromItem(tag);
	}

	public void readFromItem(NBTTagCompound tag) {
		if(tag.hasKey("itemRot")){
			itemRot = tag.getInteger("itemRot");
		}
		
		if (tag.hasKey("name")) {
			name = tag.getString("name");
		}

		if (tag.hasKey("red")) {
			red = tag.getInteger("red");
		}

		if (tag.hasKey("green")) {
			green = tag.getInteger("green");
		}

		if (tag.hasKey("blue")) {
			blue = tag.getInteger("blue");
		}

		if (tag.hasKey("buffer")) {
			buffer = tag.getFloat("buffer");
		}

		if (tag.hasKey("items")) {
			NBTTagCompound tags = tag.getCompoundTag("items");

			for (int i = 0; i < 6; i++) {
				if (tags.hasKey("" + i)) {
					items[i] = ItemStack.loadItemStackFromNBT(tags
							.getCompoundTag("" + i));
				}
			}
		}
		
		if(tag.hasKey("clear")){
			clear = tag.getBoolean("clear");
		}
		
		if(tag.hasKey("handle")){
			angle2 = tag.getFloat("handle");
		}
	}

	@Override
	public void writeToNBT(NBTTagCompound tag) {
		super.writeToNBT(tag);
		writeToItem(tag);
	}

	public void writeToItem(NBTTagCompound tag) {
		NBTTagCompound tags = new NBTTagCompound();

		if(itemRot != 0){
			tag.setInteger("itemRot", itemRot);
		}
		
		if (name != null) {
			tag.setString("name", name);
		}

		if (red != 255) {
			tag.setInteger("red", red);
		}

		if (green != 255) {
			tag.setInteger("green", green);
		}

		if (blue != 255) {
			tag.setInteger("blue", blue);
		}

		if (buffer != 0F) {
			tag.setFloat("buffer", buffer);
		}

		boolean stacks = false;
		for (int i = 0; i < 6; i++) {
			ItemStack stack = items[i];
			if (stack != null) {
				NBTTagCompound tmp = new NBTTagCompound();
				stack.writeToNBT(tmp);
				tags.setTag("" + i, tmp);
				stacks = true;
			}
		}

		if (stacks) {
			tag.setTag("items", tags);
		}
		
		if(clear){
			tag.setBoolean("clear", true);
		}
		
		if(angle2 != 0){
			tag.setFloat("handle", angle2);
		}
	}

	@Override
	public void updateEntity() {
		lastAngle = angle;

		itemRot += 5;
		
		if(itemRot > 360){
			itemRot -= 360;
		}
		
		if (open > 0 && angle == 0.0F) {
			worldObj.playSoundEffect(xCoord + 0.5, yCoord + 0.5, zCoord + 0.5,
					"random.chestopen", 0.5F,
					worldObj.rand.nextFloat() * 0.1F + 0.9F);
		}

		if (open == 0 && angle > 0.0F || open > 0 && angle < 1.0F) {
			float f1 = angle;

			if (open > 0) {
				angle += 0.1F;
			} else {
				angle -= 0.1F;
			}

			if (angle > 1.0F) {
				angle = 1.0F;
			}

			float f2 = 0.5F;

			if (angle < f2 && f1 >= f2) {
				worldObj.playSoundEffect(xCoord + 0.5, yCoord + 0.5,
						zCoord + 0.5, "random.chestclosed", 0.5F,
						worldObj.rand.nextFloat() * 0.1F + 0.9F);
			}

			if (angle < 0.0F) {
				angle = 0.0F;
			}
		}
		
		lastAngle2 = angle2;
		
		if (angle2 < 1.0F) {
			angle2 += 0.05F;

			if (angle2 > 1.0F) {
				angle2 = 1.0F;
			}
		}
	}

	@Override
	public int getSizeInventory() {
		return 6;
	}

	@Override
	public ItemStack getStackInSlot(int slot) {
		return items[slot];
	}

	@Override
	public ItemStack decrStackSize(int slot, int count) {
		if (items[slot] != null) {
			if (items[slot].stackSize <= count) {
				ItemStack stack = items[slot];
				items[slot] = null;
				onInventoryChanged();
				return stack;
			} else {
				ItemStack stack = items[slot].splitStack(count);
				if (items[slot].stackSize == 0) {
					items[slot] = null;
				}

				onInventoryChanged();
				return stack;
			}
		}

		return null;
	}

	@Override
	public ItemStack getStackInSlotOnClosing(int slot) {
		if (items[slot] != null) {
			ItemStack stack = items[slot];
			items[slot] = null;
			return stack;
		}

		return null;
	}

	@Override
	public void setInventorySlotContents(int slot, ItemStack stack) {
		items[slot] = stack;

		if (items[slot] != null
				&& items[slot].stackSize > getInventoryStackLimit()) {
			items[slot].stackSize = getInventoryStackLimit();
		}

		onInventoryChanged();
	}

	@Override
	public String getInvName() {
		return name == null ? "gud_cooler:inv.cooler.name" : name;
	}

	@Override
	public boolean isInvNameLocalized() {
		return name != null;
	}

	@Override
	public int getInventoryStackLimit() {
		return 64;
	}

	@Override
	public boolean isUseableByPlayer(EntityPlayer player) {
		return worldObj.getBlockTileEntity(xCoord, yCoord, zCoord) != this ? false
				: player.getDistanceSq((double) xCoord + 0.5D,
						(double) yCoord + 0.5D, (double) zCoord + 0.5D) <= 64.0D;
	}

	@Override
	public void openChest() {
		if (open < 0) {
			open = 0;
		}

		open++;
		worldObj.addBlockEvent(xCoord, yCoord, zCoord, getBlockType().blockID,
				1, open);
	}

	@Override
	public void closeChest() {
		open--;
		worldObj.addBlockEvent(xCoord, yCoord, zCoord, getBlockType().blockID,
				1, open);
	}

	@Override
	public boolean receiveClientEvent(int type, int value) {
		if (type == 1) {
			open = value;
			return true;
		}

		return super.receiveClientEvent(type, value);
	}

	@Override
	public boolean isItemValidForSlot(int slot, ItemStack stack) {
		if (stack != null && stack.getItem() != null) {
			return stack.getItem() instanceof ItemFood;
		}

		return false;
	}

	private static final int[] side = {};

	@Override
	public int[] getAccessibleSlotsFromSide(int var1) {
		return side;
	}

	@Override
	public boolean canInsertItem(int i, ItemStack itemstack, int j) {
		return false;
	}

	@Override
	public boolean canExtractItem(int i, ItemStack itemstack, int j) {
		return false;
	}

	public Packet getDescriptionPacket() {
		NBTTagCompound tag = new NBTTagCompound();
		writeToItem(tag);
		return new Packet132TileEntityData(xCoord, yCoord, zCoord, 0, tag);
	}

	public void onDataPacket(INetworkManager netManager,
			Packet132TileEntityData packet) {
		readFromItem(packet.data);
		worldObj.markBlockForUpdate(packet.xPosition, packet.yPosition,
				packet.zPosition);
	}

	public ItemStack[] getInventory() {
		return items;
	}
}
